==> closest_pairs-0.1.3.tar.gz: closest_pairs-0.1.3/closest_pairs/main.py <==
import math

import numpy as np
from sklearn.decomposition import PCA


def solution(array: np.ndarray, n=1):
    """Solve the closest pairs problem.
    Note: Code borrowed from Andriy Lazorenko's Medium post.
    """
    if array.shape[1] > 2:
        array = reduce_dims(array)

    pairs = []
    mis = []

    x, y = np.split(array, 2, axis=1)
    for i in range(n):
        a = list(zip(x, y))  # This produces list of tuples
        ax = sorted(a, key=lambda x: x[0])  # Presorting x-wise
        ay = sorted(a, key=lambda x: x[1])  # Presorting y-wise
        p1, p2, mi = closest_pair(ax, ay)  # Recursive D&C function

        p1_arg = np.where(array[:, 0] == p1[0])[0]
        p2_arg = np.where(array[:, 0] == p2[0])[0]

        pairs.append((p1_arg, p2_arg))
        mis.append(mi)

        # Remove point from array
        x = np.delete(x, p1_arg)
        y = np.delete(y, p1_arg)

    return np.unique(np.array(pairs),axis=1), np.array(mis)


def reduce_dims(array: np.ndarray):
    pca = PCA(n_components=2)
    array_2d = pca.fit(array).transform(array)

    return array_2d


def closest_pair(ax: np.ndarray, ay: np.ndarray):
    """Find the closest pair.
    Note: Code borrowed from Andriy Lazorenko.
    """
    ln_ax = len(ax)  # It's quicker to assign variable
    if ln_ax <= 3:
        return brute(ax)  # A call to bruteforce comparison
    mid = ln_ax // 2  # Division without remainder, need int
    Qx = ax[:mid]  # Two-part split
    Rx = ax[mid:]
    # Determine midpoint on x-axis
    midpoint = ax[mid][0]
    Qy = list()
    Ry = list()
    for x in ay:  # split ay into 2 arrays using midpoint
        if x[0] <= midpoint:
            Qy.append(x)
        else:
            Ry.append(x)
    # Call recursively both arrays after split
    (p1, q1, mi1) = closest_pair(Qx, Qy)
    (p2, q2, mi2) = closest_pair(Rx, Ry)
    # Determine smaller distance between points of 2 arrays
    if mi1 <= mi2:
        d = mi1
        mn = (p1, q1)
    else:
        d = mi2
        mn = (p2, q2)
    # Call function to account for points on the boundary
    (p3, q3, mi3) = closest_split_pair(ax, ay, d, mn)
    # Determine smallest distance for the array
    if d <= mi3:
        return mn[0], mn[1], d
    else:
        return p3, q3, mi3


def brute(ax: np.ndarray):
    mi = dist(ax[0], ax[1])
    p1 = ax[0]
    p2 = ax[1]
    ln_ax = len(ax)
    if ln_ax == 2:
        return p1, p2, mi
    for i in range(ln_ax - 1):
        for j in range(i + 1, ln_ax):
            if i != 0 and j != 1:
                d = dist(ax[i], ax[j])
                if d < mi:  # Update min_dist and points
                    mi = d
                    p1, p2 = ax[i], ax[j]
    return p1, p2, mi


def dist(p1: np.ndarray, p2: np.ndarray):
    return math.sqrt((p1[0] - p2[0]) ** 2 + (p1[1] - p2[1]) ** 2)


def closest_split_pair(p_x: list, p_y: np.ndarray, delta: float, best_pair: tuple):
    """Find the closest_split_pair.
    Note: Code modified from Andriy Lazorenko.
    """
    ln_x = len(p_x)  # store length - quicker
    mx_x = p_x[ln_x // 2][0]  # select midpoint on x-sorted array
    # Create a subarray of points not further than delta from
    # midpoint on x-sorted array
    s_y = [x for x in p_y if mx_x - delta <= x[0] <= mx_x + delta]
    best = delta  # assign best value to delta
    ln_y = len(s_y)  # store length of subarray for quickness
    for i in range(ln_y - 1):
        for j in range(i + 1, min(i + 7, ln_y)):
            p, q = s_y[i], s_y[j]
            dst = dist(p, q)
            if dst < best:
                best_pair = p, q
                best = dst
    return best_pair[0], best_pair[1], best

==> closest_pairs-0.1.4.tar.gz: closest_pairs-0.1.4/closest_pairs/main.py <==
import math

import numpy as np
from sklearn.decomposition import PCA


def solution(array: np.ndarray, n=1):
    """Solve the closest pairs problem.
    Note: Code borrowed from Andriy Lazorenko's Medium post.
    """
    if array.shape[1] > 2:
        array = reduce_dims(array)

    pairs = []
    mis = []

    x, y = np.split(array, 2, axis=1)
    for i in range(n):
        a = list(zip(x, y))  # This produces list of tuples
        ax = sorted(a, key=lambda x: x[0])  # Presorting x-wise
        ay = sorted(a, key=lambda x: x[1])  # Presorting y-wise
        p1, p2, mi = closest_pair(ax, ay)  # Recursive D&C function

        p1_arg = np.where(array[:, 0] == p1[0])[0]
        p2_arg = np.where(array[:, 0] == p2[0])[0]

        pairs.append((p1_arg, p2_arg))
        mis.append(mi)

        # Remove point from array
        x = np.delete(x, p1_arg)
        y = np.delete(y, p1_arg)

    return np.unique(np.array(pairs),axis=1).squeeze(), np.array(mis)


def reduce_dims(array: np.ndarray):
    pca = PCA(n_components=2)
    array_2d = pca.fit(array).transform(array)

    return array_2d


def closest_pair(ax: np.ndarray, ay: np.ndarray):
    """Find the closest pair.
    Note: Code borrowed from Andriy Lazorenko.
    """
    ln_ax = len(ax)  # It's quicker to assign variable
    if ln_ax <= 3:
        return brute(ax)  # A call to bruteforce comparison
    mid = ln_ax // 2  # Division without remainder, need int
    Qx = ax[:mid]  # Two-part split
    Rx = ax[mid:]
    # Determine midpoint on x-axis
    midpoint = ax[mid][0]
    Qy = list()
    Ry = list()
    for x in ay:  # split ay into 2 arrays using midpoint
        if x[0] <= midpoint:
            Qy.append(x)
        else:
            Ry.append(x)
    # Call recursively both arrays after split
    (p1, q1, mi1) = closest_pair(Qx, Qy)
    (p2, q2, mi2) = closest_pair(Rx, Ry)
    # Determine smaller distance between points of 2 arrays
    if mi1 <= mi2:
        d = mi1
        mn = (p1, q1)
    else:
        d = mi2
        mn = (p2, q2)
    # Call function to account for points on the boundary
    (p3, q3, mi3) = closest_split_pair(ax, ay, d, mn)
    # Determine smallest distance for the array
    if d <= mi3:
        return mn[0], mn[1], d
    else:
        return p3, q3, mi3


def brute(ax: np.ndarray):
    mi = dist(ax[0], ax[1])
    p1 = ax[0]
    p2 = ax[1]
    ln_ax = len(ax)
    if ln_ax == 2:
        return p1, p2, mi
    for i in range(ln_ax - 1):
        for j in range(i + 1, ln_ax):
            if i != 0 and j != 1:
                d = dist(ax[i], ax[j])
                if d < mi:  # Update min_dist and points
                    mi = d
                    p1, p2 = ax[i], ax[j]
    return p1, p2, mi


def dist(p1: np.ndarray, p2: np.ndarray):
    return math.sqrt((p1[0] - p2[0]) ** 2 + (p1[1] - p2[1]) ** 2)


def closest_split_pair(p_x: list, p_y: np.ndarray, delta: float, best_pair: tuple):
    """Find the closest_split_pair.
    Note: Code modified from Andriy Lazorenko.
    """
    ln_x = len(p_x)  # store length - quicker
    mx_x = p_x[ln_x // 2][0]  # select midpoint on x-sorted array
    # Create a subarray of points not further than delta from
    # midpoint on x-sorted array
    s_y = [x for x in p_y if mx_x - delta <= x[0] <= mx_x + delta]
    best = delta  # assign best value to delta
    ln_y = len(s_y)  # store length of subarray for quickness
    for i in range(ln_y - 1):
        for j in range(i + 1, min(i + 7, ln_y)):
            p, q = s_y[i], s_y[j]
            dst = dist(p, q)
            if dst < best:
                best_pair = p, q
                best = dst
    return best_pair[0], best_pair[1], best
